#ifndef __GRAPH_H__
#define __GRAPH_H__

#include "Node.h"

class Graph{

private:
	vector<Node*> nodes;
	vector<Node*>::iterator rootNodeIterator;
	
public:
	int nrNodes;

	Graph(string fileName){
		readData(fileName);
	}

	void readData(string fileName){
		std::ifstream f(fileName.c_str());
		int nrArcs;

		f >> nrNodes;
		createNodes(nrNodes);
        f >> nrArcs;
		
        int src = -1;
		int dest= -1;
		for (int i = 0; i < nrArcs; i++) {
			f >> src;
			f >> dest;				
		
			Node* srcNode = nodes.at(src);
			Node* destNode = nodes.at(dest);
			
			srcNode->addNeighbour(destNode, OUT);
			destNode->addNeighbour(srcNode, IN);				
		}
	}

	void createNodes(int nrNodes) {
		for(int i = 0; i < nrNodes; i++){
			nodes.push_back(new Node(i));
		}
	}

	Node* getNextRootNode(){		
		if ( rootNodeIterator != nodes.end() ) {
			Node* node = (*rootNodeIterator);
			rootNodeIterator++;
			if ( node->isRootNode() ){
				return node;
			} else {
				return getNextRootNode();
			}
		}
		return NULL;
	}
	
	vector<Node*> getAllRootNodes(){
		vector<Node*> rootNodes;
		for(vector<Node*>::iterator it = nodes.begin(); it != nodes.end(); ++it){
			if ( (*it)->isRootNode() )
				rootNodes.push_back(*it);
		}
		return rootNodes;
	}

	void resetRootIterator(){
		rootNodeIterator = nodes.begin();
	}
	
	
	void print(){
		for(vector<Node*>::iterator it = nodes.begin(); it != nodes.end(); ++it){
			(*it)->print();
			cout << " ";
		}
		cout << "\n";
	}


	Node* getNodeWithId(int id) {
		for(vector<Node*>::iterator it = nodes.begin(); it != nodes.end(); ++it){
			if ( (*it)->getId() == id )
				return *it;
		}
		return NULL;
	}
};


#endif